@@ -1,6 +1,6 @@
 /*
  * MONA
- * Copyright (C) 1997-2000 BRICS.
+ * Copyright (C) 1997-2002 BRICS.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
  * USA.
  */
 
+#include <iostream>
 #include <stdlib.h>
 #include <string.h>
 #include "ast.h"
@@ -25,6 +26,8 @@
 #include "predlib.h"
 #include "lib.h"
 
+using std::cout;
+
 extern PredicateLib predicateLib;
 extern SymbolTable symbolTable;
 extern CodeTable *codeTable;
@@ -36,6 +39,7 @@ extern int numTypes;
 //////////  Restriction ///////////////////////////////////////////////////////
 
 static IdentList restrVars; // variables occuring in current restriction
+static bool indefault = false; // inside defaultwhere
 
 VarCode
 getRestriction(Ident id, SubstCode *subst)
@@ -69,10 +73,13 @@ getRestriction(Ident id, SubstCode *subst)
     
     // add user defined or default restriction
     ASTForm *restriction = symbolTable.lookupRestriction(id);
-    if (restriction)
+    if (restriction) {
       // add user-defined restriction
       vc = andList(vc, restriction->makeCode(subst));
-    else if (!symbolTable.lookupImplicit(id)) { // try default if not implicit variable
+    }
+    else if (!symbolTable.lookupImplicit(id) &&
+	     !indefault) { // try default if not implicit variable and not in default
+      indefault = true;
       switch (symbolTable.lookupType(id)) {
       case Varname1:
       case Parname1:
@@ -95,10 +102,11 @@ getRestriction(Ident id, SubstCode *subst)
 	vc = andList(vc, restriction->makeCode(subst));
 	delete[] subst;
       }
+      indefault = false;
     }
-    
+
     restrVars.remove(id);
-  }
+  }   
   return vc;
 }
 
@@ -135,6 +143,7 @@ shiftStatespaces(IdentList *univs, int dir)
   }
   return res;
 }
+/*#warning NEW: STATESPACE SHIFTING*/
 
 //////////  Substitution //////////////////////////////////////////////////////
 
@@ -216,6 +225,7 @@ substitute0Code(SubstCode *subst, Ident var)
     }
   }
 
+  /*#warning NEW: BUGFIX AT VAR0*/
   VarCode r = getRestriction(var, subst);
   
   // restrict if not trivial restriction
@@ -408,7 +418,7 @@ ASTTerm1_Root::makeCode(SubstCode *subst)
     exit(-1);
   }
   univ = d2->get(0);
-  if (symbolTable.lookupType(univ)!=Univname) {
+  if (symbolTable.lookupType(univ)!=Univname && symbolTable.lookupType(univ)!=Statespacename) {
     cout << "Error:";
     pos.printsource();
     cout << "\nIllegal argument to 'root'\n"
@@ -783,6 +793,7 @@ ASTTerm2_Inter::makeCode(SubstCode *subst)
 
   IdentList *univs = ident_union(symbolTable.lookupUnivs(ct1->var),
 				 symbolTable.lookupUnivs(ct2->var));
+  /*#warning CHANGED TO UNION (BECAUSE OF STATESPACES AS UNIVS)*/
 
   Ident var = symbolTable.insertFresh(Varname2, univs);
 
@@ -1454,6 +1465,7 @@ ASTForm_All2::makeCode(SubstCode *subst)
   return res;
 }
 
+/*#warning LET0/1/2 REWRITTEN*/
 VarCode
 ASTForm_Let0::makeCode(SubstCode *subst)
 {
@@ -1569,6 +1581,8 @@ ASTForm_Let2::makeCode(SubstCode *subst)
   return res;
 }
 
+/*#warning BUGFIX: REMOVED UPDATEUNIVS*/
+
 VarCode
 ASTForm_Call::makeCode(SubstCode *subst)
 {
