@@ -1,6 +1,6 @@
 /*
  * MONA
- * Copyright (C) 1997-2000 BRICS.
+ * Copyright (C) 1997-2002 BRICS.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1146,8 +1146,8 @@ public:
   ASTForm *formula;
   ASTForm *assertion;
 
-  Deque<ASTForm *> verifyformlist;
-  Deque<char *>    verifytitlelist;
+  DequeGC<ASTForm *> verifyformlist;
+  Deque<char *>      verifytitlelist;
 
   IdentList globals; // all globally declared variables
   Ident lastPosVar;  // lastpos variable
