@@ -1,6 +1,6 @@
 /*
  * MONA
- * Copyright (C) 1997-2000 BRICS.
+ * Copyright (C) 1997-2002 BRICS.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * USA.
  */
 
-#include <iostream.h>
+#include <iostream>
 #include <string.h>
 #include "codetable.h"
 #include "offsets.h"
@@ -27,6 +27,8 @@
 #include "env.h"
 #include "lib.h"
 
+using std::cout;
+
 extern "C" {
 #include "../Mem/mem.h"
 }
@@ -62,6 +64,21 @@ stateSpaces(IdentList *univs) // find state spaces for univs
       case Statespacename:
 	set[symbolTable.lookupNumber(*id)] = 1;
 	break;
+
+      case Varname1:
+      case Varname2:
+      case Parname1:
+      case Parname2:
+      case VarnameTree:
+	// inherit universes from the variable
+	{
+	  SSSet s = stateSpaces(*id);
+	  for (unsigned i = 0; i < guide.numSs; i++)
+	    set[i] = set[i] | s[i];
+	  mem_free(s);
+	  break;
+	}
+	
       default: 
 	invariant(0);
       }
@@ -369,6 +386,8 @@ Code_c::Code_c(CodeKind knd, VarCode c, Pos p, bool skipinit) :
     sign.make(vars);
     vars.compress();
   }
+  if (vc.code)
+    depth = vc.code->depth+1;
 }
 
 bool
@@ -401,6 +420,11 @@ Code_cc::Code_cc(CodeKind knd, VarCode c1, VarCode c2, Pos p):
   vars.append(vc2.vars);
   sign.make(vars);
   vars.compress();
+
+  if (vc1.code->depth>vc2.code->depth)
+    depth = vc1.code->depth+1;
+  else
+    depth = vc2.code->depth+1;  
 }
 
 bool
@@ -418,6 +442,112 @@ Code_cc::hash()
   return kind*2 + ((int) vc1.code)*5 + ((int) vc2.code)*7;
 }
 
+void 
+Code_cc::makeDFA()
+{
+  /* #warning NEW: heuristic choice through DAG - 1-15% lower max-aut. */
+  if (vc1.code->refs==1 || (vc2.code->refs>1 && vc1.code->depth<=vc2.code->depth)) {
+    a1 = vc1.DFATranslate();
+    vc1.remove();
+    a2 = vc2.DFATranslate();
+    vc2.remove();
+  }
+  else {
+    a2 = vc2.DFATranslate();
+    vc2.remove();
+    a1 = vc1.DFATranslate();
+    vc1.remove();
+  }
+}
+
+void 
+Code_cc::makeGTA()
+{
+  if (vc1.code->refs==1 || (vc2.code->refs>1 && vc1.code->depth<=vc2.code->depth)) {
+    g1 = vc1.GTATranslate();
+    vc1.remove();
+    g2 = vc2.GTATranslate();
+    vc2.remove();
+  }
+  else {
+    g2 = vc2.GTATranslate();
+    vc2.remove();
+    g1 = vc1.GTATranslate();
+    vc1.remove();
+  }
+}
+
+/**
+////////// Code_clist /////////////////////////////////////////////////////////
+
+Code_clist::Code_clist(CodeKind knd, Deque<VarCode> c, Pos p):
+  Code(knd, p), vcl(c) 
+{
+  Deque<VarCode>::iterator i;
+  int d = 0;
+  / * REQUIRE SIZE>0 * /
+  
+  if (kind==cAnd || kind==cOr || kind==cBiimpl) // exploit symmetry
+    / * SORT!   if (vc1.code < vc2.code) {
+      VarCode t = vc1;
+      vc1 = vc2;
+      vc2 = t;
+      }* /;
+
+  for (i = vcl.begin(); i != vcl.end(); i++) {
+    vars.append(i->vars);
+    if (i->code->depth > d)
+      d = i->code->depth+1;
+  }
+
+  sign.make(vars);
+  vars.compress();
+}
+
+bool
+Code_clist::equiv(Code &c)
+{
+  Deque<VarCode>::iterator i, j;
+  
+  if (kind != c.kind ||
+      !(sign == ((Code_clist&) c).sign) ||
+      vcl.size() != ((Code_clist&) c).vcl.size()) 
+    return 0;
+  
+  for (i = vcl.begin(), j = ((Code_clist&) c).vcl.begin(); i != vcl.end(); i++, j++)
+    if (!(i->code == j->code))
+      return 0;
+  
+  return 1;
+}
+
+unsigned 
+Code_clist::hash() 
+{
+  return kind*2 + vcl.size()*7;
+}
+
+void 
+Code_clist::makeDFA()
+{
+  Deque<VarCode>::iterator i;
+  for (i = vcl.begin(); i != vcl.end(); i++) {
+    al.push_back(i->DFATranslate());
+    i->remove();
+  }
+}
+
+void 
+Code_clist::makeGTA()
+{
+  Deque<VarCode>::iterator i;
+  for (i = vcl.begin(); i != vcl.end(); i++) {
+    gl.push_back(i->GTATranslate());
+    i->remove();
+  }
+}
+**/
+
 ////////// Atomic formulas ////////////////////////////////////////////////////
 
 void
@@ -694,7 +824,7 @@ void
 Code_EqUnion::makeGTA()
 {
   gta = gtaUnion(offsets.off(id1), offsets.off(id2), offsets.off(id3),
-		 stateSpaces(id1));
+		 stateSpaces(id1), stateSpaces(id2), stateSpaces(id3));
 }
 
 void 
@@ -707,7 +837,7 @@ void
 Code_EqInter::makeGTA()
 {
   gta = gtaInter(offsets.off(id1), offsets.off(id2), offsets.off(id3),
-		 stateSpaces(id1));
+		 stateSpaces(id1), stateSpaces(id2), stateSpaces(id3));
 }
 
 void
@@ -720,7 +850,7 @@ void
 Code_EqSetMinus::makeGTA()
 {  
   gta = gtaSetminus(offsets.off(id1), offsets.off(id2), offsets.off(id3),
-		    stateSpaces(id1));
+		    stateSpaces(id1), stateSpaces(id2), stateSpaces(id3));
 }
 
 void
@@ -752,14 +882,14 @@ Code_WellFormedTree::makeGTA()
 void 
 Code_Restrict::makeDFA()
 {
-  dfa = st_dfa_restrict(vc.DFATranslate(), pos);
+  dfa = st_dfa_minimization(st_dfa_restrict(vc.DFATranslate(), pos));
   vc.remove();
 }
 
 void 
 Code_Restrict::makeGTA()
 {
-  gta = st_gta_restrict(vc.GTATranslate(), pos);
+  gta = st_gta_minimization(st_gta_restrict(vc.GTATranslate(), pos));
   vc.remove();
 }
 
@@ -839,20 +969,14 @@ Code_Prefix::makeDFA()
 void 
 Code_And::makeDFA()
 {
-  DFA *a1 = vc1.DFATranslate();
-  vc1.remove();
-  DFA *a2 = vc2.DFATranslate();
-  vc2.remove();
+  Code_cc::makeDFA();
   dfa = st_dfa_minimization(st_dfa_product(a1, a2, dfaAND, pos));
 }
 
 void 
 Code_And::makeGTA()
 {
-  GTA *g1 = vc1.GTATranslate();
-  vc1.remove();
-  GTA *g2 = vc2.GTATranslate();
-  vc2.remove();
+  Code_cc::makeGTA();
   gta = st_gta_minimization(st_gta_product(g1, g2, gtaAND, pos));
 }
 
@@ -861,10 +985,7 @@ Code_And::makeGTA()
 void 
 Code_IdLeft::makeDFA()
 {
-  DFA *a1 = vc1.DFATranslate();
-  vc1.remove();
-  DFA *a2 = vc2.DFATranslate();
-  vc2.remove();
+  Code_cc::makeDFA();
   dfaFree(a2);
   dfa = a1;
 }
@@ -872,10 +993,7 @@ Code_IdLeft::makeDFA()
 void 
 Code_IdLeft::makeGTA()
 {
-  GTA *g1 = vc1.GTATranslate();
-  vc1.remove();
-  GTA *g2 = vc2.GTATranslate();
-  vc2.remove();
+  Code_cc::makeGTA();
   gtaFree(g2);
   gta = g1;
 }
@@ -885,20 +1003,14 @@ Code_IdLeft::makeGTA()
 void 
 Code_Or::makeDFA()
 {
-  DFA *a1 = vc1.DFATranslate();
-  vc1.remove();
-  DFA *a2 = vc2.DFATranslate();
-  vc2.remove();
+  Code_cc::makeDFA();
   dfa = st_dfa_minimization(st_dfa_product(a1, a2, dfaOR, pos));
 }
 
 void 
 Code_Or::makeGTA()
 {
-  GTA *g1 = vc1.GTATranslate();
-  vc1.remove();
-  GTA *g2 = vc2.GTATranslate();
-  vc2.remove();
+  Code_cc::makeGTA();
   gta = st_gta_minimization(st_gta_product(g1, g2, gtaOR, pos));
 }
 
@@ -907,20 +1019,14 @@ Code_Or::makeGTA()
 void 
 Code_Impl::makeDFA()
 {
-  DFA *a1 = vc1.DFATranslate();
-  vc1.remove();
-  DFA *a2 = vc2.DFATranslate();
-  vc2.remove();
+  Code_cc::makeDFA();
   dfa = st_dfa_minimization(st_dfa_product(a1, a2, dfaIMPL, pos));
 }
 
 void 
 Code_Impl::makeGTA()
 {
-  GTA *g1 = vc1.GTATranslate();
-  vc1.remove();
-  GTA *g2 = vc2.GTATranslate();
-  vc2.remove();
+  Code_cc::makeGTA();
   gta = st_gta_minimization(st_gta_product(g1, g2, gtaIMPL, pos));
 }
 
@@ -929,20 +1035,14 @@ Code_Impl::makeGTA()
 void 
 Code_Biimpl::makeDFA()
 {
-  DFA *a1 = vc1.DFATranslate();
-  vc1.remove();
-  DFA *a2 = vc2.DFATranslate();
-  vc2.remove();
+  Code_cc::makeDFA();
   dfa = st_dfa_minimization(st_dfa_product(a1, a2, dfaBIIMPL, pos));
 }
 
 void 
 Code_Biimpl::makeGTA()
 {
-  GTA *g1 = vc1.GTATranslate();
-  vc1.remove();
-  GTA *g2 = vc2.GTATranslate();
-  vc2.remove();
+  Code_cc::makeGTA();
   gta = st_gta_minimization(st_gta_product(g1, g2, gtaBIIMPL, pos));
 }
 
