@@ -1,6 +1,6 @@
 /*
  * MONA
- * Copyright (C) 1997-2000 BRICS.
+ * Copyright (C) 1997-2002 BRICS.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 #include "st_gta.h"
 #include "printline.h"
 #include "ident.h"
-#include "string.h"
+#include "str.h"
 
 ////////// StateSpaces ////////////////////////////////////////////////////////
 
@@ -44,7 +44,7 @@ enum CodeKind {
   cEqMinus1, cEqPlus2, cEqMinus2, cEqPlusModulo, cEqMinusModulo,
   cEqMin, cEqMax, cAnd, cOr, cImpl, cBiimpl, cRestrict, cProject,
   cNegate, cImport, cExport, cPrefix, cInStateSpace, cIdLeft, 
-  cEqPresbConst, cWellFormedTree, cSomeType
+  cEqPresbConst, cWellFormedTree, cSomeType, cAndList
 };
 
 class Code;
@@ -113,7 +113,7 @@ class Code {
 public:
   Code(CodeKind knd, Pos p) :
     kind(knd), refs(1), pos(p), mark(0), eqlist(NULL), dfa(NULL), gta(NULL),
-    conj(NULL), restrconj(NULL) {}
+    conj(NULL), restrconj(NULL), depth(0)/**, conjhash(0)**/ {}
   virtual ~Code() {}
 
   // determine syntax/signature equivalence
@@ -140,6 +140,9 @@ public:
   virtual void setmark(int val);
   virtual void clearEqlist();
   virtual bool checkExport(Ident x);
+/**
+  void hashconj();
+**/
 
   // copying (codesubst.cpp)
   virtual VarCode substCopy(IdentList *actuals) = 0;
@@ -161,6 +164,10 @@ public:
   GTA       *gta;          // default GTA, NULL if not generated yet
   VarCodeList *conj;       // conjuncts (used during red.)
   VarCodeList *restrconj;  // restricted conjuncts (used during red.)
+  int        depth;        // max number of steps to leaf
+/**
+  unsigned   conjhash;     // hashing of conj and restrconj (used during red.)
+**/
 };
 
 ////////// Abstract classes ///////////////////////////////////////////////////
@@ -250,11 +257,39 @@ public:
   void clearEqlist();
   bool checkExport(Ident x);
   void show() {};
+  virtual void makeDFA();
+  virtual void makeGTA();
 
   VarCode vc1;
   VarCode vc2;
+  DFA *a1,*a2;
+  GTA *g1,*g2;
 };
 
+/**
+class Code_clist: public Code {
+public:
+  Code_clist(CodeKind knd, Deque<VarCode> c, Pos p);
+  ~Code_clist() {for (Deque<VarCode>::iterator i = vcl.begin(); i != vcl.end(); i++) i->remove();}
+
+  bool equiv(Code&);
+  unsigned hash();
+  void setmark(int val);
+  void viz();
+  void reduce1();
+  virtual void reduce2();
+  void clearEqlist();
+  bool checkExport(Ident x);
+  void show() {};
+  virtual void makeDFA();
+  virtual void makeGTA();
+
+  Deque<VarCode> vcl;
+  Deque<DFA*> al;
+  Deque<GTA*> gl;
+};
+**/
+
 ////////// Atomic formulas ////////////////////////////////////////////////////
 
 class Code_True: public Code {
@@ -672,6 +707,21 @@ public:
   VarCode substCopy(IdentList *actuals);
 };
 
+/*
+class Code_AndList: public Code_cc {
+public:
+  Code_AndList(Deque<VarCode> c, Pos p) :
+    Code_clist(cAndList, c, p) {}
+
+  void makeDFA();
+  void makeGTA();
+  void dump(bool rec);
+  void reduce1();
+  void reduce2();
+  VarCode substCopy(IdentList *actuals);
+};
+*/
+
 class Code_IdLeft: public Code_cc {
 public:
   Code_IdLeft(VarCode vc1, VarCode vc2, Pos p) :
