@@ -1,6 +1,6 @@
 /*
  * MONA
- * Copyright (C) 1997-2000 BRICS.
+ * Copyright (C) 1997-2002 BRICS.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,10 +18,15 @@
  * USA.
  */
 
+/*
+#warning NEW: EXPLICIT RESTRICT PREDICATE
+#warning NEW: SEPARATE VERIFICATION
+*/
+
 #define _LANGUAGE_C_PLUS_PLUS
 
-#include <iostream.h>
-#include <new.h>
+#include <iostream>
+#include <new>
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <signal.h>
@@ -37,11 +42,14 @@
 #include "timer.h"
 #include "lib.h"
 #include "printline.h"
+#include "config.h"
 
 extern "C" {
 #include "../Mem/mem.h"
 }
 
+using std::cout;
+
 Options options;
 MonaUntypedAST *untypedAST;
 SymbolTable symbolTable(1019);
@@ -67,7 +75,7 @@ ParseArguments(int argc, char *argv[])
   options.printProgress = true;
   options.analysis = true;
   options.optimize = 1;
-  options.reorder = true;
+  options.reorder = false; //true;
 
   switch (argc) {
   case 1:
@@ -190,8 +198,8 @@ ParseArguments(int argc, char *argv[])
 void 
 Usage()
 {
-  cout << "MONA v" << VERSION << " for WS1S/WS2S\n"
-    << "Copyright (C) 1997-2000 BRICS\n\n"
+  cout << "MONA v" << VERSION << "-" << RELEASE << " for WS1S/WS2S\n"
+    << "Copyright (C) 1997-2002 BRICS\n\n"
     << "Usage: mona [options] <filename>\n\n"
     << "Options:\n"
     << " -w   Output whole automaton\n"
@@ -202,7 +210,8 @@ Usage()
     << " -d   Dump AST, symboltable, and code DAG\n"
     << " -q   Quiet, don't print progress\n\n"
     << " -e   Enable separate compilation\n"
-    << " -oN  Code optimization level N (default 1)\n"
+    << " -oN  Code optimization level N (0=none, 1=safe, 2=heuristic) (default 1)\n"
+//  << " -r   Disable BDD index reordering\n"
     << " -f   Force normal tree-mode output style\n"
     << " -m   Alternative M2L-Str emulation (v1.3 style)\n"
     << " -h   Inherited acceptance analysis\n"
@@ -218,18 +227,19 @@ Usage()
     << "Full documentation is available at http://www.brics.dk/mona\n";
 }
 
-void
+RETSIGTYPE
 cpuLimit(int)
 {
   cout << "\n\n-----\n"
        << "Interactive Demo time exceeded, execution stopped.\n";
   exit(-1);
+  SIGRET;
 }
 
 int 
 main(int argc, char *argv[])
 {
-  set_new_handler(&mem_error);
+  std::set_new_handler(&mem_error);
 
   if (!ParseArguments(argc, argv)) {
     Usage();
@@ -265,8 +275,8 @@ main(int argc, char *argv[])
   ///////// PARSING ////////////////////////////////////////////////////////
 
   if (options.printProgress)
-    cout << "MONA v" << VERSION " for WS1S/WS2S\n"
-      "Copyright (C) 1997-2000 BRICS\n\n"
+    cout << "MONA v" << VERSION << "-" << RELEASE <<  " for WS1S/WS2S\n"
+      "Copyright (C) 1997-2002 BRICS\n\n"
       "PARSING\n";
 
   Timer timer_parsing;
@@ -295,7 +305,7 @@ main(int argc, char *argv[])
     Deque<char *>::iterator vt;
     for (vf = ast->verifyformlist.begin(), vt = ast->verifytitlelist.begin();
 	 vf != ast->verifyformlist.end(); vf++, vt++) {
-      cout << "\n\nVerify " << *vt << ":\n";
+      cout << "\n\nFormula " << *vt << ":\n";
       (*vf)->dump();
     }
     cout << "\n\nAssertions:\n";
@@ -379,6 +389,7 @@ main(int argc, char *argv[])
   VarCode formulaCode = ast->formula->makeCode();
   VarCode assertionCode = ast->assertion->makeCode();
   Deque<VarCode> verifyCode;
+  /* #warning NEW: 'VERIFY' */
   for (Deque<ASTForm *>::iterator i = ast->verifyformlist.begin(); 
        i != ast->verifyformlist.end(); i++)
     verifyCode.push_back((*i)->makeCode());
@@ -398,7 +409,6 @@ main(int argc, char *argv[])
        i != verifyCode.end(); i++) {
     assertionCode.code->refs++;
     *i = andList(*i, VarCode(copy(assertionCode.vars), assertionCode.code));
-    (*i).code->refs++;
   }
   formulaCode = andList(formulaCode, assertionCode);
 
@@ -432,6 +442,7 @@ main(int argc, char *argv[])
     for (Deque<VarCode>::iterator i = verifyCode.begin(); 
 	 i != verifyCode.end(); i++)
       newverifycode.push_back((*i).substCopy(&emptylist, &emptylist));
+    codeTable->clearSCTable();
     regenerate = false;
     codeTable = oldCodeTable;
     formulaCode.remove();
@@ -479,9 +490,17 @@ main(int argc, char *argv[])
     symbolTable.dump();
     
     // Dump code
-    cout << "\nFormula code:\n";
+    cout << "\nMain formula:\n";
     formulaCode.dump();
     cout << "\n\n";
+    Deque<VarCode>::iterator i;
+    Deque<char *>::iterator j;
+    for (i = verifyCode.begin(), j = ast->verifytitlelist.begin(); 
+	 i != verifyCode.end(); i++, j++) {
+      cout << "Formula " << *j << ":\n";
+      (*i).dump();
+      cout << "\n\n";
+    }
   }
   
   if (options.graphvizDAG) {
@@ -491,7 +510,19 @@ main(int argc, char *argv[])
 	   " main -> L%x;\n", 
 	   (unsigned) formulaCode.code);
     formulaCode.code->viz();
+    Deque<VarCode>::iterator i;
+    Deque<char *>::iterator j;
+    for (i = verifyCode.begin(), j = ast->verifytitlelist.begin(); 
+	 i != verifyCode.end(); i++, j++) {
+      printf(" \"%s\" [shape = plaintext];\n"
+	     " \"%s\" -> L%x;\n", 
+	     *j, *j, (unsigned) (*i).code);
+      (*i).code->viz();
+    }
     formulaCode.unmark();
+    for (Deque<VarCode>::iterator i = verifyCode.begin(); 
+	 i != verifyCode.end(); i++)
+      (*i).unmark();
     cout << "}\n";
   }
 
@@ -871,4 +902,7 @@ main(int argc, char *argv[])
     cout << "\nTotal time: ";
     timer_total.print();
   }
+#ifdef MAXALLOCATED
+  cout << "Maximum space allocated: " << (maxallocated+524288)/1048576 << " MB\n";
+#endif
 }
