@@ -1,6 +1,6 @@
 /*
  * MONA
- * Copyright (C) 1997-2000 BRICS.
+ * Copyright (C) 1997-2002 BRICS.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,9 @@
 #include <stdlib.h>
 #include "offsets.h"
 
+#include <time.h>
+/*#warning USING RANDOM ORDERING*/
+
 void
 Offsets::insert()
 {
@@ -33,5 +36,18 @@ Offsets::reorder()
 {
   ////// reorder offsets using BDD heuristics ///////
 
+  /*
+#warning USING RANDOM ORDERING
+  ///////////// test /////////
+  
+  srandom(time(0));
+  for (unsigned i=0; i<offsetMap.size(); i++) {
+    unsigned j = ((unsigned) random()) % offsetMap.size();
+    unsigned t = offsetMap.get(j);
+    offsetMap.set(j, offsetMap.get(i));
+    offsetMap.set(i, t);
+  }
+  ////////////////////////////
+  */
   max_offset = offsetMap.size();
 }
