@@ -1,6 +1,6 @@
 /*
  * MONA
- * Copyright (C) 1997-2000 BRICS.
+ * Copyright (C) 1997-2002 BRICS.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,8 @@ extern "C" {
 #include "../GTA/gta.h"
 }
 
+using std::cout;
+
 extern SymbolTable symbolTable;
 extern PredicateLib predicateLib;
 extern Options options; 
@@ -1481,11 +1483,11 @@ AST*
 UntypedExp_WellFormedTree::genAST()
 {
   if (numTypes == 0)
-    TypeError("'well_formed_tree' predicate only allowed when using types", pos);
+    TypeError("'tree' predicate only allowed when using types", pos);
 
   AST *t = exp->genAST();
   if (t->order != oTerm2)
-    TypeError("Type mismatch at 'well_formed_tree'", pos);
+    TypeError("Type mismatch at 'tree'", pos);
 
   return new ASTForm_WellFormedTree((ASTTerm2 *) t, pos);
 }
@@ -2018,7 +2020,7 @@ MonaUntypedAST::typeCheck()
 
     if (options.mode == TREE) {
       // m2l-tree --> tree;
-      //              var2 $ where all1 p: (p in $) => ((p^ in $) | (p^=p));
+      //              var2 $ where all1 p where true: (p in $) => ((p^ in $) | (p^=p));
       //              allpos $;
       //              defaultwhere1(p) = p in $;
       //              defaultwhere2(P) = P sub $;
@@ -2043,7 +2045,8 @@ MonaUntypedAST::typeCheck()
       declarations->push_front
 	(new AllPos_Declaration(new Name(vardol, dummyPos), dummyPos));
 
-      // var2 $ where all1 p: (p in $) => ((p^ in $) | (p^=p));
+      // var2 $ where all1 p where true: (p in $) => ((p^ in $) | (p^=p));
+/*#warning bugfix: 'where true'*/
       exp1 = new UntypedExp_Name(new Name(varp, dummyPos), dummyPos);
       exp1 = new UntypedExp_Up(exp1, dummyPos);
       exp2 = new UntypedExp_Name(new Name(vardol, dummyPos), dummyPos);
@@ -2064,7 +2067,8 @@ MonaUntypedAST::typeCheck()
       exp = new UntypedExp_Impl(exp1, exp, dummyPos);
 
       vlist = new VarDeclList(); 
-      vlist->push_back(new VarDecl(new Name(varp, dummyPos), NULL, dummyPos));
+      vlist->push_back(new VarDecl(new Name(varp, dummyPos), 
+				   new UntypedExp_True(dummyPos), dummyPos));
       exp = new UntypedExp_All1(NULL, vlist, exp, dummyPos);
 
       vlist = new VarDeclList(); 
