@@ -1,6 +1,6 @@
 /*
  * MONA
- * Copyright (C) 1997-2000 BRICS.
+ * Copyright (C) 1997-2002 BRICS.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,12 +24,14 @@
 #include "../Mem/mem.h"
 #include "gta.h"
 
+/*
 static SSSet copySSSet(SSSet s)
 {
   SSSet r = (SSSet) mem_alloc(guide.numSs);
   memcpy(r, s, guide.numSs);
   return r;
 }
+*/
 
 GTA *gtaIn(int P, int Q, SSSet uP, SSSet uQ)
 { 
@@ -163,14 +165,13 @@ GTA *gtaFirstOrder(int P, SSSet uP) {
   return gtaBuild("--+"); /* Met is the only accepting state */
 }
 
-/* gtaAllPos accepts if and only if there is an occurrence of
+/* gtaLastPos accepts if and only if there is an occurrence of
    variable P in every leaf in every non-empty universe in uP;
    the condition that the universe be non-empty is derived
    from our use below of the state Initial;it is not possible
    currently to use initial states that depend on the universe,
    thus we are forced to accept an empty tree in a universe in uP */
-/** should be changed to be similar to gtaAllPos */
-GTA *gtaAllPos(int P, SSSet uP) { 
+GTA *gtaLastPos(int P, SSSet uP) { 
   int var[1], d, Initial = 0, Bad = 1, Met=2; 
 /* Met in hat transition is accepting */
 
@@ -221,7 +222,70 @@ GTA *gtaAllPos(int P, SSSet uP) {
     gtaBuildDelta(Initial); 
   }
   mem_free(uP);
-  return gtaBuild("+0+"); /* Met is the only accepting state */
+  return gtaBuild("+0+");
+}
+
+/* gtaAllPos accepts if and only if there is an occurrence of
+   variable P in every node in every universe in uP */
+GTA *gtaAllPos(int P, SSSet uP) { 
+  int var[1], d, Initial = 0, Bad = 1, Met=2; 
+
+  var[0] = P;
+  
+  gtaSetup(3); /* 3 states in root */
+  
+  for (d = 0; d < guide.numSs; d++) {
+    gtaSetupDelta(d, 3, 3, var, 1); /* 3 states in all state spaces */
+
+    gtaAllocExceptions(Bad, Bad, 0);
+    gtaStoreDefault(Bad);
+    
+    gtaAllocExceptions(Bad, Initial, 0);
+    gtaStoreDefault(Bad);
+    
+    gtaAllocExceptions(Initial, Bad, 0);
+    gtaStoreDefault(Bad);
+    
+    gtaAllocExceptions(Met, Bad, 0);
+    gtaStoreDefault(Bad);
+    
+    gtaAllocExceptions(Bad, Met, 0);
+    gtaStoreDefault(Bad);
+    
+    if (hasMember(uP, d)) {
+      gtaAllocExceptions(Initial,Initial,1);
+      gtaStoreException(Met, "1");
+      gtaStoreDefault(Bad); 
+
+      gtaAllocExceptions(Met, Met, 1);
+      gtaStoreException(Met, "1");
+      gtaStoreDefault(Bad); 
+      
+      gtaAllocExceptions(Met, Initial, 1);
+      gtaStoreException(Met, "1");
+      gtaStoreDefault(Bad); 
+      
+      gtaAllocExceptions(Initial, Met, 1);
+      gtaStoreException(Met, "1");
+      gtaStoreDefault(Bad); 
+    }
+    else {
+      gtaAllocExceptions(Initial,Initial, 0);
+      gtaStoreDefault(Initial); 
+      
+      gtaAllocExceptions(Met, Met, 0);
+      gtaStoreDefault(Met);
+      
+      gtaAllocExceptions(Met, Initial, 0);
+      gtaStoreDefault(Met);
+      
+      gtaAllocExceptions(Initial, Met, 0);
+      gtaStoreDefault(Met);
+    }    
+    gtaBuildDelta(Initial); 
+  }
+  mem_free(uP);
+  return gtaBuild("+0+");
 }
 
 GTA *gtaEq1(int P, int Q, SSSet uP, SSSet uQ)
@@ -514,7 +578,7 @@ GTA *gtaDot1(int P, int Q, SSSet uP, SSSet uQ)
 	gtaStoreException(NoP, "X1");
 	gtaStoreDefault(Bad);
       }
-      else { /* d not member of uPQ */
+      else {
 	gtaAllocExceptions(NoP, NoP, 0);
 	gtaStoreDefault(NoP); 
 	
@@ -598,7 +662,7 @@ GTA *gtaDot0(int P, int Q, SSSet uP, SSSet uQ)
 	gtaStoreException(NoP, "X1");
 	gtaStoreDefault(Bad);
       }
-      else { /* d not member of uPQ */
+      else {
 	gtaAllocExceptions(NoP, NoP, 0);
 	gtaStoreDefault(NoP); 
 	
@@ -614,6 +678,8 @@ GTA *gtaDot0(int P, int Q, SSSet uP, SSSet uQ)
   }
 }
 
+/*#warning NEW: GTAUP, GTADOT0, AND GTADOT1 CHANGED*/
+
 GTA *gtaUp(int P, int Q, SSSet uP, SSSet uQ)
 {
   /* Calculate the automaton that checks that the positions in P are 
@@ -769,7 +835,7 @@ GTA *gtaUp(int P, int Q, SSSet uP, SSSet uQ)
 	gtaStoreDefault(Bad); /* children are not roots */
 	
       }
-      else { /* d not member of uPQ */
+      else {
 	gtaAllocExceptions(Ok, Ok, 0);
 	gtaStoreDefault(Ok);
 	
@@ -1202,6 +1268,7 @@ GTA *gtaBoolvar(int alpha)
   return gtaBuild("+-");
 }
 
+/*#warning NEW: gtaBoolvar2*/
 /* should use statespaces! */
 GTA *gtaBoolvar2(int alpha)
 {
@@ -1269,15 +1336,21 @@ GTA *gtaBoolvar2(int alpha)
   return gtaBuild("0+-0");
 }
 
-/* We assume common universes again */
-GTA *gtaUnion(int P, int Q, int R, SSSet uPQR)
+/*#warning BUGFIX: NO LONGER ASSUMING COMMON UNIVERSES*/
+GTA *gtaUnion(int P, int Q, int R, SSSet uP, SSSet uQ, SSSet uR)
 {
-  if (P == Q) 
-    return gtaSub(R, P, uPQR, copySSSet(uPQR));
-  else if (P == R) 
-    return gtaSub(Q, R, uPQR, copySSSet(uPQR));
-  else if (Q == R)
-    return gtaEq2(P, Q, uPQR, copySSSet(uPQR));
+  if (P == Q) {
+    mem_free(uQ);
+    return gtaSub(R, P, uR, uP);
+  }
+  else if (P == R) {
+    mem_free(uP);
+    return gtaSub(Q, R, uQ, uR);
+  }
+  else if (Q == R) {
+    mem_free(uR);
+    return gtaEq2(P, Q, uP, uQ);
+  }
   else {
     int var[3], d, Ok = 0, Bad = 1;
     
@@ -1288,18 +1361,50 @@ GTA *gtaUnion(int P, int Q, int R, SSSet uPQR)
     gtaSetup(2); /* Two states in root */
     for (d = 0; d < guide.numSs; d++) {
       gtaSetupDelta(d, 2, 2, var, 3); /* 2 states in all state spaces */
-      if (hasMember(uPQR, d)) {
+      if (hasMember(uP,d) && hasMember(uQ,d) && hasMember(uR,d)) {
 	gtaAllocExceptions(Ok, Ok, 3);
 	gtaStoreException(Ok, "1X1");
 	gtaStoreException(Ok, "110");
 	gtaStoreException(Ok, "000");
 	gtaStoreDefault(Bad);
       }
+      else if (hasMember(uP,d) && hasMember(uQ,d) && !hasMember(uR,d)) {
+	gtaAllocExceptions(Ok, Ok, 2);
+	gtaStoreException(Ok, "11X");
+	gtaStoreException(Ok, "00X");
+	gtaStoreDefault(Bad);
+      }
+      else if (hasMember(uP,d) && !hasMember(uQ,d) && hasMember(uR,d)) {
+	gtaAllocExceptions(Ok, Ok, 2);
+	gtaStoreException(Ok, "1X1");
+	gtaStoreException(Ok, "0X0");
+	gtaStoreDefault(Bad);
+      }
+      else if (hasMember(uP,d) && !hasMember(uQ,d) && !hasMember(uR,d)) {
+	gtaAllocExceptions(Ok, Ok, 1);
+	gtaStoreException(Ok, "0XX");
+	gtaStoreDefault(Bad);
+      }
+      else if (!hasMember(uP,d) && hasMember(uQ,d) && hasMember(uR,d)) {
+	gtaAllocExceptions(Ok, Ok, 1);
+	gtaStoreException(Ok, "X00");
+	gtaStoreDefault(Bad);
+      }
+      else if (!hasMember(uP,d) && hasMember(uQ,d) && !hasMember(uR,d)) {
+	gtaAllocExceptions(Ok, Ok, 1);
+	gtaStoreException(Ok, "X0X");
+	gtaStoreDefault(Bad);
+      }
+      else if (!hasMember(uP,d) && !hasMember(uQ,d) && hasMember(uR,d)) {
+	gtaAllocExceptions(Ok, Ok, 1);
+	gtaStoreException(Ok, "XX0");
+	gtaStoreDefault(Bad);
+      }
       else {
 	gtaAllocExceptions(Ok, Ok, 0);
 	gtaStoreDefault(Ok);
       }
-      
+
       gtaAllocExceptions(Ok, Bad, 0);
       gtaStoreDefault(Bad);
       
@@ -1311,20 +1416,28 @@ GTA *gtaUnion(int P, int Q, int R, SSSet uPQR)
       
       gtaBuildDelta(Ok); 
     }
-    mem_free(uPQR);
+    mem_free(uP);
+    mem_free(uQ);
+    mem_free(uR);
     return gtaBuild("+-");
   }
 }
 
-/* We assume common universes again */
-GTA *gtaInter(int P, int Q, int R, SSSet uPQR)
+/*#warning BUGFIX: NO LONGER ASSUMING COMMON UNIVERSES*/
+GTA *gtaInter(int P, int Q, int R, SSSet uP, SSSet uQ, SSSet uR)
 {
-  if (P == Q) 
-    return gtaSub(P, R, uPQR, copySSSet(uPQR));
-  else if (P == R)
-    return gtaSub(P, Q, uPQR, copySSSet(uPQR));
-  else if (Q == R)
-    return gtaEq2(P, Q, uPQR, copySSSet(uPQR));
+  if (P == Q) {
+    mem_free(uQ);
+    return gtaSub(P, R, uP, uR);
+  }
+  else if (P == R) {
+    mem_free(uR);
+    return gtaSub(P, Q, uP, uQ);
+  }
+  else if (Q == R) {
+    mem_free(uR);
+    return gtaEq2(P, Q, uP, uQ);
+  }
   else {
     int var[3], d, Ok = 0, Bad = 1;
     
@@ -1335,18 +1448,41 @@ GTA *gtaInter(int P, int Q, int R, SSSet uPQR)
     gtaSetup(2); /* Two states in root */
     for (d = 0; d < guide.numSs; d++) {
       gtaSetupDelta(d, 2, 2, var, 3); /* 2 states in all state spaces */
-      if (hasMember(uPQR, d)) {
+      if (hasMember(uP,d) && hasMember(uQ,d) && hasMember(uR,d)) {
 	gtaAllocExceptions(Ok, Ok, 3);
 	gtaStoreException(Ok, "111");
 	gtaStoreException(Ok, "00X");
 	gtaStoreException(Ok, "010");
 	gtaStoreDefault(Bad);
-      }
+      } 
+      else if (hasMember(uP,d) && hasMember(uQ,d) && !hasMember(uR,d)) {
+	gtaAllocExceptions(Ok, Ok, 1);
+	gtaStoreException(Ok, "0XX");
+	gtaStoreDefault(Bad);
+      } 
+      else if (hasMember(uP,d) && !hasMember(uQ,d) && hasMember(uR,d)) {
+	gtaAllocExceptions(Ok, Ok, 2);
+	gtaStoreException(Ok, "1X1");
+	gtaStoreException(Ok, "0XX");
+	gtaStoreDefault(Bad);
+      } 
+      else if (hasMember(uP,d) && !hasMember(uQ,d) && !hasMember(uR,d)) {
+	gtaAllocExceptions(Ok, Ok, 1);
+	gtaStoreException(Ok, "0XX");
+	gtaStoreDefault(Bad);
+      } 
+      else if (!hasMember(uP,d) && hasMember(uQ,d) && hasMember(uR,d)) {
+	gtaAllocExceptions(Ok, Ok, 3);
+	gtaStoreException(Ok, "X11");
+	gtaStoreException(Ok, "X0X");
+	gtaStoreException(Ok, "X10");
+	gtaStoreDefault(Bad);
+      } 
       else {
 	gtaAllocExceptions(Ok, Ok, 0);
 	gtaStoreDefault(Ok);
       }
-      
+
       gtaAllocExceptions(Ok, Bad, 0);
       gtaStoreDefault(Bad);
       
@@ -1358,17 +1494,22 @@ GTA *gtaInter(int P, int Q, int R, SSSet uPQR)
       
       gtaBuildDelta(Ok); 
     }
-    mem_free(uPQR);
+    mem_free(uP);
+    mem_free(uQ);
+    mem_free(uR);
     return gtaBuild("+-");
   }  
 }
 
-/* We assume common universes again */
-GTA *gtaSetminus(int P, int Q, int R, SSSet uPQR)
+/*#warning BUGFIX: NO LONGER ASSUMING COMMON UNIVERSES*/
+GTA *gtaSetminus(int P, int Q, int R, SSSet uP, SSSet uQ, SSSet uR)
 {
   if (P == R) {
-    if (P == Q)
-      return gtaEmpty(P, uPQR);
+    if (P == Q) {
+      mem_free(uQ);
+      mem_free(uR);
+      return gtaEmpty(P, uP);
+    }
     else {
       /* P == R and P != Q, so P = Q \ R means that P and Q are empty */
       int var[2], d, Ok = 0, Bad = 1;
@@ -1378,12 +1519,20 @@ GTA *gtaSetminus(int P, int Q, int R, SSSet uPQR)
       gtaSetup(2); 
       for (d = 0; d < guide.numSs; d++) {
 	gtaSetupDelta(d, 2, 2, var, 3); 
-	if (hasMember(uPQR, d)) {
- 	  gtaAllocExceptions(Ok, Ok, 2);
- 	  gtaStoreException(Ok, "00");
+	if (hasMember(uP,d) && hasMember(uQ,d)) {
+ 	  gtaAllocExceptions(Ok, Ok, 1);
  	  gtaStoreException(Ok, "00");
  	  gtaStoreDefault(Bad);
-	}
+	} else if (hasMember(uP,d) && !hasMember(uQ,d)) {
+ 	  gtaAllocExceptions(Ok, Ok, 1);
+ 	  gtaStoreException(Ok, "0X");
+ 	  gtaStoreDefault(Bad);
+	} 
+	else if (!hasMember(uP,d) && hasMember(uQ,d)) {
+ 	  gtaAllocExceptions(Ok, Ok, 1);
+ 	  gtaStoreException(Ok, "X0");
+ 	  gtaStoreDefault(Bad);
+	} 
 	else {
 	  gtaAllocExceptions(Ok, Ok, 0);
 	  gtaStoreDefault(Ok);
@@ -1400,15 +1549,20 @@ GTA *gtaSetminus(int P, int Q, int R, SSSet uPQR)
 	
    	gtaBuildDelta(Ok); 
       }
-      mem_free(uPQR);
+      mem_free(uP);
+      mem_free(uQ);
+      mem_free(uR);
       return gtaBuild("+-");
     }
   }
   else
     /* P != R */
 
-    if (Q == R)
-      return gtaEmpty(P, uPQR);
+    if (Q == R) {
+      mem_free(uQ);
+      mem_free(uR);
+      return gtaEmpty(P, uP);
+    }
     else {
       if (P == Q) { /* P inter R = empty */
 	int var[2], d, Ok = 0, Bad = 1;
@@ -1419,11 +1573,11 @@ GTA *gtaSetminus(int P, int Q, int R, SSSet uPQR)
 	gtaSetup(2); 
 	for (d = 0; d < guide.numSs; d++) {
 	  gtaSetupDelta(d, 2, 2, var, 2); /* 2 states in all state spaces */
-	  if (hasMember(uPQR, d)) {
-	    gtaAllocExceptions(Ok, Ok, 1);
+	  if (hasMember(uP,d) && hasMember(uR,d)) {
+ 	    gtaAllocExceptions(Ok, Ok, 1);
 	    gtaStoreException(Bad, "11");
 	    gtaStoreDefault(Ok);
-	  }
+	  } 
 	  else {
 	    gtaAllocExceptions(Ok, Ok, 0);
 	    gtaStoreDefault(Ok);
@@ -1440,7 +1594,9 @@ GTA *gtaSetminus(int P, int Q, int R, SSSet uPQR)
 	  
 	  gtaBuildDelta(Ok); 
 	}
-	mem_free(uPQR);
+	mem_free(uP);
+	mem_free(uQ);
+	mem_free(uR);
 	return gtaBuild("+-");
       } else { /* P, Q, R: they are all different */
 	int var[3], d, Ok = 0, Bad = 1;
@@ -1452,15 +1608,43 @@ GTA *gtaSetminus(int P, int Q, int R, SSSet uPQR)
 	gtaSetup(2); 
 	for (d = 0; d < guide.numSs; d++) {
 	  gtaSetupDelta(d, 2, 2, var, 3); /* 2 states in all state spaces */
-	  if (hasMember(uPQR, d)) {
+	  if (hasMember(uP,d) && hasMember(uQ,d) && hasMember(uR,d)) {
 	    gtaAllocExceptions(Ok, Ok, 3);
 	    gtaStoreException(Ok, "110");
 	    gtaStoreException(Ok, "00X");
 	    gtaStoreException(Ok, "011");
 	    gtaStoreDefault(Bad);
-	  }
+	  } 
+	  else if (hasMember(uP,d) && hasMember(uQ,d) && !hasMember(uR,d)) {
+	    gtaAllocExceptions(Ok, Ok, 2);
+	    gtaStoreException(Ok, "11X");
+	    gtaStoreException(Ok, "00X");
+	    gtaStoreDefault(Bad);
+	  } 
+	  else if (hasMember(uP,d) && !hasMember(uQ,d) && hasMember(uR,d)) {
+	    gtaAllocExceptions(Ok, Ok, 2);
+	    gtaStoreException(Ok, "0XX");
+	    gtaStoreException(Ok, "0X1");
+	    gtaStoreDefault(Bad);
+	  } 
+	  else if (hasMember(uP,d) && !hasMember(uQ,d) && !hasMember(uR,d)) {
+	    gtaAllocExceptions(Ok, Ok, 1);
+	    gtaStoreException(Ok, "0XX");
+	    gtaStoreDefault(Bad);
+	  } 
+	  else if (!hasMember(uP,d) && hasMember(uQ,d) && hasMember(uR,d)) {
+	    gtaAllocExceptions(Ok, Ok, 2);
+	    gtaStoreException(Ok, "X0X");
+	    gtaStoreException(Ok, "X11");
+	    gtaStoreDefault(Bad);
+	  } 
+	  else if (!hasMember(uP,d) && hasMember(uQ,d) && !hasMember(uR,d)) {
+	    gtaAllocExceptions(Ok, Ok, 1);
+	    gtaStoreException(Ok, "X0X");
+	    gtaStoreDefault(Bad);
+	  } 
 	  else {
-	    gtaAllocExceptions(Ok, Ok, 0);
+    	    gtaAllocExceptions(Ok, Ok, 0);
 	    gtaStoreDefault(Ok);
 	  }
 	  
@@ -1475,12 +1659,16 @@ GTA *gtaSetminus(int P, int Q, int R, SSSet uPQR)
 	  
 	  gtaBuildDelta(Ok); 
 	}
-	mem_free(uPQR);
+	mem_free(uP);
+	mem_free(uQ);
+	mem_free(uR);
 	return gtaBuild("+-");
       }
     }
 }
 
+/*#warning NEW: GTAINSTATESPACE CHANGED*/
+
 GTA *gtaInStateSpace(int P, SSSet ss, SSSet uP)
 { 
   int var[1], d, Ok = 0, Bad = 1;
